﻿-----------------------------------------------------------------------------
--
-- data
--   \dispel
--     \<playerName>
--       - count
--
-----------------------------------------------------------------------------

local L = AceLibrary("AceLocale-2.2"):new("AssessmentDispel")
L:RegisterTranslations("enUS", function() return {
	["Dispellers List"] = true,
	["Dispels"] = true,

	["Dispel"] = true,
	["Dispel Options."] = true,
	["Gather Direct Only"] = true,
	["Only gather direct dispels.\nThis prevents warrior's shield bash and breaking crowd control as counting towards dispels."] = true,
} end)

L:RegisterTranslations("koKR", function() return {
	["Dispellers List"] = "해제 목록",
	["Dispels"] = "해제",

	["Dispel"] = "해제",
	["Dispel Options."] = "해제 설정입니다.",
	["Gather Direct Only"] = "직접적인 것만 수집",
	["Only gather direct dispels.\nThis prevents warrior's shield bash and breaking crowd control as counting towards dispels."] = "직접 해제 가능한 것만 수집합니다.\n전사의 방패 가격과 군중 제어 파괴가 해제로 계산되는 것을 방지합니다.",
} end)

-----------------------------------------------------------------------------

local AssessmentDispel = Assessment:NewModule("Dispel", "AceEvent-2.0", "Parser-3.0")
AssessmentDispel.db = Assessment:AcquireDBNamespace("Dispel")
Assessment:RegisterDefaults("Dispel", "profile", {
	enable = true,
	gatherDirectOnly = true,
})

-----------------------------------------------------------------------------

local profile
local emptyMethod = Assessment.EmptyMethod
local new = Assessment.NewTable
local pairs = pairs

-----------------------------------------------------------------------------

Assessment.TagData.DispelCount =
{
	method = function(data)
				local fightData = data.fightData
				if not fightData then return 0 end
				
				local dispel = fightData.dispel
				if not dispel then return 0 end
				
				local playerDispel = dispel[data.key]
				if not playerDispel then return 0 end

				return playerDispel.count
			 end,
	data = "dispel",
	guaranteeResult = true,
	numeric = true,
}

-----------------------------------------------------------------------------

Assessment.ListType.Dispellers =
{
	method = function(fightData)
				if not fightData or not fightData.dispel then return emptyMethod end
				return pairs(fightData.dispel)
			 end,
	data = "dispel",
	description = L["Dispellers List"],
}

-----------------------------------------------------------------------------

Assessment.DefaultDisplaySets.Dispels =
{
	titleTag		= L["Dispels"],
	listType		= "Dispellers", 
	barValueTag		= "DispelCount", 
	filterTag		= "BarValue ~= 0",
	fightIndex		= 0,
	description		= L["Dispels"],
	defaultColor	= { 0.5, 0.3, 0.8 },
	colorTypeList	= { "Class", count = 1 },
}

-----------------------------------------------------------------------------

Assessment.Options.args.data.args.dispel =
{
	type = "group",
	name = L["Dispel"],
	desc = L["Dispel Options."],
	args =
	{
		gatherDirectOnly =
		{
			type = "toggle",
			name = L["Gather Direct Only"],
			desc = L["Only gather direct dispels.\nThis prevents warrior's shield bash and breaking crowd control as counting towards dispels."],
			get = function() return profile.gatherDirectOnly end,
			set = function(v) profile.gatherDirectOnly = v end,
		},
	}
}

-----------------------------------------------------------------------------

function AssessmentDispel:OnInitialize()
	profile = self.db.profile
end

function AssessmentDispel:OnEnable()
	self:RegisterParserEvent({ eventType = "Dispel" }, "OnParserDispelEvent")
end

function AssessmentDispel:OnDisable()
end

-----------------------------------------------------------------------------

function AssessmentDispel:OnParserDispelEvent(eventData)
	if not eventData.sourceName or eventData.isFailed then return end

	if profile.gatherDirectOnly then
		if not eventData.sourceID ~= not eventData.recipientID then
			return
		end
	end

	local time = GetTime()
	for data in self:IterateUpdateDataTables() do
		data.dataUpdateTimes.dispel = time
		local dispel = data.dispel
		if not dispel then
			dispel = new()
			data.dispel = dispel
		end

		local sourceName = eventData.sourceName
		local playerDispel = dispel[sourceName]
		if playerDispel then
			playerDispel.count = playerDispel.count + 1
		else
			playerDispel = new()
			playerDispel.count = 1
			dispel[sourceName] = playerDispel
		end
	end
end

-----------------------------------------------------------------------------
