﻿-----------------------------------------------------------------------------
--
-- data
--   \offline
--     \<playerName>
--       - count, duration
--
-----------------------------------------------------------------------------

local L = AceLibrary("AceLocale-2.2"):new("AssessmentOffline")
L:RegisterTranslations("enUS", function() return {
	["Offline List"] = true,
	["Offline"] = true,
} end)

L:RegisterTranslations("koKR", function() return {
	["Offline List"] = "접속종료 목록",
	["Offline"] = "접속종료",
} end)

L:RegisterTranslations("frFR", function() return {
	["Offline List"] = "Liste des D\195\169connect\195\169s",
	["Offline"] = "D\195\169connect\195\169",
} end)

local Roster = AceLibrary("Roster-2.1")

-----------------------------------------------------------------------------

local AssessmentOffline = Assessment:NewModule("Offline", "AceEvent-2.0")
AssessmentOffline.db = Assessment:AcquireDBNamespace("Offline")
Assessment:RegisterDefaults("Offline", "profile", {
	enable	= true,
})

-----------------------------------------------------------------------------

local profile

local emptyMethod = Assessment.EmptyMethod
local new, del, copy, reset = Assessment.NewTable, Assessment.ReleaseTable, Assessment.CopyTable, Assessment.ResetTable
local pairs = pairs

-----------------------------------------------------------------------------

Assessment.TagData.OfflineDuration =
{
	method = function(data)
				local fightData = data.fightData
				if not fightData then return 0 end
				
				local offline = fightData.offline
				if not offline then return 0 end
				
				local playerOffline = offline[data.key]
				if not playerOffline then return 0 end

				return playerOffline.duration
			 end,
	data = "offline",
	guaranteeResult = true,
	numeric = true,
}

Assessment.TagData.OfflineCount =
{
	method = function(data)
				local fightData = data.fightData
				if not fightData then return 0 end
				
				local offline = fightData.offline
				if not offline then return 0 end
				
				local playerOffline = offline[data.key]
				if not playerOffline then return 0 end

				return playerOffline.count
			 end,
	data = "offline",
	guaranteeResult = true,
	numeric = true,
}

-----------------------------------------------------------------------------

Assessment.ListType.OfflineList =
{
	method = function(fightData)
				if not fightData or not fightData.offline then return emptyMethod end
				return pairs(fightData.offline)
			 end,
	data = "offline",
	description = L["Offline List"],
}

-----------------------------------------------------------------------------

Assessment.DefaultDisplaySets.Offline =
{
	titleTag			= L["Offline"],
	listType			= "OfflineList", 
	barValueTag			= "OfflineDuration", 
	filterTag			= "1",
	fightIndex			= 0,
	text				= { count = 2, { tag = "[Rank]. [Key] ([OfflineCount])" }, { tag = "[BarValue:FormatDuration]" } },
	description			= L["Offline"],
	defaultColor		= { 0.7, 0.7, 0.7 },
	colorTypeList		= { "Class", count = 1 },
}

-----------------------------------------------------------------------------

Assessment.ListType.EnumerateTable.updateData.offline = "offline"

-----------------------------------------------------------------------------

local offlineList = { }

-----------------------------------------------------------------------------

function AssessmentOffline:OnInitialize()
	profile = self.db.profile
end

function AssessmentOffline:OnEnable()
	self:RegisterEvent("RosterLib_RosterUpdated", "OnRosterUpdated")
end

function AssessmentOffline:OnDisable()
end

-----------------------------------------------------------------------------

function AssessmentOffline:OnRosterUpdated()
	local oldOfflineList = copy(offlineList)
	reset(offlineList)

	for unit in Roster:IterateRoster() do
		local name = unit.name
		
		if not unit.online then
			if not oldOfflineList[name] then
				offlineList[name] = true
				
				local time = GetTime()
				for data in self:IterateOOCUpdateDataTables() do
					data.dataUpdateTimes.offline = time

					local offline = data.offline
					if not offline then
						offline = new()
						data.offline = offline
					end

					local playerOffline = offline[name]
					if playerOffline then
						playerOffline.count = playerOffline.count + 1
						playerOffline.durationOffset = playerOffline.duration-time
					else
						playerOffline = new()
						playerOffline.count = 1
						playerOffline.duration = 0
						playerOffline.durationOffset = -time
						offline[name] = playerOffline
					end
				end
			end
		end
		oldOfflineList[name] = nil
	end
	
	for name in pairs(oldOfflineList) do
		offlineList[name] = nil
	end
	del(oldOfflineList)
	
	if next(offlineList) then
		self:ScheduleRepeatingEvent("UpdateOfflineDurations", self.UpdateOfflineDurations, 0.1, self)
		self:RegisterEvent("Assessment:NewFightDataTable", "OnNewFightDataTable")
	else
		self:CancelScheduledEvent("UpdateOfflineDurations")
		self:CancelEvent("Assessment:NewFightDataTable")
	end
end

function AssessmentOffline:OnNewFightDataTable(data)
	local time = GetTime()
	data.dataUpdateTimes.offline = time

	local offline = new()
	data.offline = offline

	for name in pairs(offlineList) do
		local playerOffline = new()
		playerOffline.count = 1
		playerOffline.duration = 0
		playerOffline.durationOffset = -time
		offline[name] = playerOffline
	end
end

function AssessmentOffline:UpdateOfflineDurations()
	local time = GetTime()

	for data in self:IterateOOCUpdateDataTables() do
		data.dataUpdateTimes.offline = time
		local offline = data.offline
		for name in pairs(offlineList) do
			local playerOffline = offline[name]
			playerOffline.duration = playerOffline.durationOffset + time
		end
	end
end

-----------------------------------------------------------------------------
